# -*- coding: utf-8 -*-
from bobtemplates.plone.base import base_prepare_renderer
from bobtemplates.plone.base import git_commit
from bobtemplates.plone.base import is_string_in_file
from bobtemplates.plone.base import set_global_vars as base_set_global_vars
from bobtemplates.plone.base import update_file


def _update_setup_py(configurator):
    file_name = u"setup.py"
    file_path = configurator.variables["package.root_folder"] + "/" + file_name
    match_str = "-*- Extra requirements: -*-"
    insert_strings = [
        "collective.transmogrifier",
        "plone.app.transmogrifier",
        "transmogrify.dexterity",
        "ftw.blueprints",
        "collective.jsonmigrator",
    ]
    for insert_str in insert_strings:
        insert_str = "        '{0}',\n".format(insert_str)
        if is_string_in_file(configurator, file_path, insert_str):
            continue
        update_file(configurator, file_path, match_str, insert_str)

    match_str = "target = plone"
    insert_strings = [
        "run_migration = {0}.migration.scripts:run_migration".format(
            configurator.variables["package.dottedname"]
        ),
        "list_migrations = {0}.migration.scripts:list_migrations".format(
            configurator.variables["package.dottedname"]
        ),
        "[zopectl.command]",
    ]
    for insert_str in insert_strings:
        insert_str = "    {0}\n".format(insert_str)
        if is_string_in_file(configurator, file_path, insert_str):
            continue
        update_file(configurator, file_path, match_str, insert_str)


def _update_configure_zcml(configurator):
    file_name = u"configure.zcml"
    file_path = configurator.variables["package_folder"] + "/" + file_name

    match_str = "-*- extra stuff goes here -*-"
    insert_str = """
  <include package=".migration" />
"""
    update_file(configurator, file_path, match_str, insert_str)


def prepare_renderer(configurator):
    configurator = base_prepare_renderer(configurator)
    configurator.target_directory = configurator.variables["package_folder"]


def post_renderer(configurator):
    _update_setup_py(configurator)
    _update_configure_zcml(configurator)
    git_commit(configurator, "Added migration")


def set_global_vars(configurator):
    base_set_global_vars(configurator)



# -*- coding: utf-8 -*-
from AccessControl.SecurityManagement import newSecurityManager
from AccessControl.SecurityManager import setSecurityPolicy
from collective.transmogrifier.transmogrifier import configuration_registry
from collective.transmogrifier.transmogrifier import Transmogrifier
from Products.CMFCore.tests.base.security import OmnipotentUser
from Products.CMFCore.tests.base.security import PermissiveSecurityPolicy
from Testing.makerequest import makerequest
from zope.site.hooks import setSite

import logging
import sys
import transaction


# Force application logging level to INFO and
# log output to stdout for all loggers
root_logger = logging.getLogger()
root_logger.setLevel(logging.INFO)

handler = logging.StreamHandler(sys.stdout)
formatter = logging.Formatter('%(asctime)s %(levelname)s %(name)s %(message)s')
handler.setFormatter(formatter)
root_logger.addHandler(handler)


def spoofRequest(app):
    """
    Make REQUEST variable to be available on the Zope application server.

    This allows acquisition to work properly
    """
    _policy = PermissiveSecurityPolicy()
    _oldpolicy = setSecurityPolicy(_policy)  # NOQA F841
    newSecurityManager(None, OmnipotentUser().__of__(app.acl_users))
    return makerequest(app)


def run_migration(self, *args):
    call = args[0]
    if len(call) > 2:
        pipeline_id = call[2]
        for plonesite in self.objectValues('Plone Site'):
            setSite(plonesite)
            plonesite = spoofRequest(plonesite)
            if pipeline_id in configuration_registry.listConfigurationIds():
                transmogrifier = Transmogrifier(plonesite)
                overrides = {}
                if len(call) >= 4:
                    json_path = call[3]
                    overrides['jsonsource'] = {'path': json_path}

                transmogrifier(pipeline_id, **overrides)
                transaction.commit()
                print('All set!')
            else:
                print('This migration id is invalid.')

    else:
        print('You need to tell me the migration id to run')
